# -- coding: utf-8 --
"""Script usado para calcular a inclinação das retas de aproximação de cada um
dos grupos segundo a definição recebida.
"""

from __future__ import division

import sys
import numpy as np

from src import *

from parsers import StpyParser

### Defines ###
CALC_HT_FAKE = 99
CALC_HB_FAKE = 99.99
CALC_MAX_ALPHA = 0.25 # max value for positive alphas
CALC_MIN_ALPHA = -0.20 # min value for negative alphas
CALC_MAX_HT = 52
CALC_MIN_HT = 21
CALC_MAX_HB = CALC_MAX_HT / 3
CALC_MIN_HB = CALC_MIN_HT / 3


def neighborhood(iterable):
    iterator = iter(iterable)
    prev = None
    item = iterator.next()  # throws StopIteration if empty.
    for next in iterator:
        yield (item, next)
        prev = item
        item = next
    yield (item, None)

def real_values (ht, hb):
    if ht == CALC_HT_FAKE and hb == CALC_HB_FAKE:
        s = "Invalid hemtocrit and hemoglobin values. Abort!"
        sys.exit(s)
    real_ht = (3*hb) if (ht == CALC_HT_FAKE) else ht
    real_hb = (ht/3) if (hb == CALC_HB_FAKE) else hb
    return (real_ht, real_hb)

def calculator(group_definition, paths):

    gd = group_definition
    parser = StpyParser(paths['files']['extracted'])

    num_donors = parser.meta['num_donors']

    meta = {
        'start_date': parser.meta['start_date'],
        'lowest_class': 0, #EXT_MIN_DONATIONS,
        'highest_class': gd['highest_donation_class'],
        'negative_threshold': CALC_MIN_ALPHA,
        'positive_threshold': CALC_MAX_ALPHA
    }
    group_classes = {} # dict with classes indexed by number of donations
    for i in range(meta['lowest_class'], meta['highest_class'] + 1):
        group_classes[i] = {} # dict with alphas indexed by donor id

    i = 1
    for donor in parser.donors():

        anemia_related = 'never'
        first_anemic_donation = None
        anemic_rate = 39.0 if donor.sex == "M" else 38.0
        for d in donor.donations:
            date_offset, ht, hb, type = d
            ht, hb = real_values(ht, hb)
            if ht < anemic_rate:
                anemia_related = 'before'
                first_anemic_donation = d
                break

        dates = []
        metric = []
        num_frac = 0
        group_start = None
        group_donations = []
        for donation, next in neighborhood(donor.donations):
            date_offset, ht, hb, type = donation

            if type != "AF":
                num_frac += 1

            # start of a group
            if not group_start:
                group_start = date_offset

            if next:
                date_offset_next = next[0]
            else:
                date_offset_next = INF
            gap = (date_offset_next - date_offset)

            # ignore donation if much close to the next
            if gap < gd['gap_min_days'] and type != "AF":
                continue
                
            ht, hb = real_values(ht, hb)
            v = ht if CALC_USE_HT_METRIC else hb
            minv = CALC_MIN_HT if CALC_USE_HT_METRIC else CALC_MIN_HB
            maxv = CALC_MAX_HT if CALC_USE_HT_METRIC else CALC_MAX_HB
            if minv <= v <= maxv:
                dates.append(date_offset)
                metric.append(v)
                group_donations.append((date_offset, v))
            
            # end of a group
            next_group_gap = (date_offset_next - group_start)
            if gap >= gd['gap_max_days'] or next_group_gap >= gd['group_interval'] \
                or id(donation) == id(first_anemic_donation):

                if len(dates) >= 2: # we need at least 2 point to do the linear regression below
                    x = np.array(dates)
                    y = np.array(metric)
                    pol = np.polyfit(x, y, 1)

                    class_id = min(gd['highest_donation_class'], num_frac)
                    groups = group_classes[class_id]
                    if donor.id not in groups.keys():
                        groups[donor.id] = []
                    groups[donor.id].append((donor.sex, anemia_related, pol, group_donations))

                if id(donation) == id(first_anemic_donation):
                    anemia_related = 'after'
                
                dates = []
                metric = []
                num_frac = 0
                group_start = None
                group_donations = []

        # progress logging
        progress_logger('Calculation', i, num_donors)
        i += 1
    sys.stdout.write('\n')
    
    fo = open(paths['files']['calculated'], 'w')
    fo.write(str(meta) + "\n")
    fs = open(paths['files']['stats'], 'w')
    for class_id, clss in group_classes.iteritems():
        data = []
        
        for donor_id, groups in clss.iteritems():
            for sex, anemia_related, pol, donations in groups:
                group_dict = {
                    'num_donations': class_id,
                    'donor_id': donor_id,
                    'pol': pol,
                    'sex': sex,
                    'anemia_related': anemia_related,
                    'donations': donations
                }
                so = str(group_dict) + '\n'
                # truncate on 2 decimals for metric value (not for alphas!)
                so = re.sub(r'(\,\s*)([0-9]+\.[0-9]{2})[0-9]+', r'\1\2', so)
                fo.write(so)
                data.append(pol[0])

        stats_dict = {
            'num_donations': class_id,
            'num_groups': len(data),
            'mean': np.average(data),
            'deviation': np.std(data)
        }
        ss = str(stats_dict) + '\n'
        fs.write(ss)
        
    fo.close()
    fs.close()
