# -- coding: utf-8 --
"""Contém todas as estruturas referentes a manipulação dos dados dentro e fora
do banco de dados.
"""

import psycopg2
from datetime import date
import re

from settings import *


class SingletonDB(object):
    meta = {}
    donor_ids = []

    # Singleton class implementation
    _instance = None
    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(SingletonDB, cls).__new__(cls, *args, **kwargs)
            cls._instance._init()
        return cls._instance

    def _init(self):
        # Definition Variables
        self.dbname = DBNAME
        self.dbuser = DBUSER
        self.dbpassword = DBPASSWORD

        # Database connection
        self._connect()

        # Extraction settings
        self.min_donations = EXT_MIN_DONATIONS
        self.donors_limit = EXT_DONORS_LIMIT

        # Metadata initialization
        q = "SELECT ano, mes, dia FROM dim_data ORDER BY ano LIMIT 1"
        rows = self.query_exec(q)
        self.meta['start_date'] = date(rows[0][0], rows[0][1], rows[0][2])
        q = "SELECT ano, mes, dia FROM dim_data ORDER BY ano DESC LIMIT 1"
        rows = self.query_exec(q)
        self.meta['end_date'] = date(rows[0][0], rows[0][1], rows[0][2])
        self.meta['num_donors'] = self._fetch_donors()


    def _connect(self):
        # Database connection
        self.conn = psycopg2.connect(database=self.dbname, user=self.dbuser,
                        password=self.dbpassword)
        self.conn.set_isolation_level(0)
        self.cur = self.conn.cursor()

    def query_exec(self, q):
        """Executa uma query e devolve a tabela resposta"""
        try:
            self.cur.execute(q)
        except psycopg2.InternalError, e:
            self._connect()
            raise e

        try:
            rows = self.cur.fetchall()
        except psycopg2.ProgrammingError, e:
            rows = []

        return rows

    def _fetch_donors(self):
        """Devolve uma lista com todos os ids de doadores por ordem de maior
        número de doações efetuadas"""

        # asserts top_doadores table was build, if not build it.
        q = "SELECT * FROM information_schema.tables WHERE table_name='top_doadores'"
        rows = self.query_exec(q)
        if not rows:
            print "Creating 'top_doadores' table..."
            q = "CREATE TABLE top_doadores (id_doador, num_doacoes) AS (SELECT id_doador, count(id_doador) FROM fato_doacao GROUP BY id_doador ORDER BY count(id_doador) DESC)"
            self.query_exec(q)

        # fetches all donors ordered by num_doacoes
        q = "SELECT id_doador, num_doacoes FROM top_doadores"
        if self.min_donations > 0:
            q += " WHERE num_doacoes >= %d" % self.min_donations
        q += " ORDER BY num_doacoes DESC"
        if self.donors_limit > 0:
            q += " LIMIT %d"  % self.donors_limit
        rows = self.query_exec(q)

        self.donor_ids = []
        total_donors = 0
        total_donations = 0
        for row in rows:
            self.donor_ids.append(row[0])
            total_donors += 1
            total_donations += row[1]

        #return total_donors, total_donations
        return total_donors

    def donations_by_donor(self, id_doador):
        """Devolve uma tabela com todas as doações feitas por um doador
        ordenadas crescentemente pela data.
        
        rows:
            [0]: f.id_doador
            [1]: f.id_data
            [2]: d.ano
            [3]: d.mes
            [4]: d.dia
            [5]: f.hematocrit
            [6]: f.hemoglobin
        """

        pdict = {'id_doador': id_doador}
        q = "SELECT f.id_doador, f.id_data, d.ano, d.mes, d.dia, f.hematocrit, f.hemoglobin, f.id_triagem, f.tipo_obtencao FROM fato_doacao f, dim_data d WHERE f.id_data = d.id_data AND f.id_doador = %(id_doador)d ORDER BY d.ano, d.mes, d.dia" % pdict
        rows = self.query_exec(q)
        return rows

    def donor_sex(self, id_doador):
        """Devolve o sexo (M ou F) conforme o sexo do doador"""
        pdict = {'id_doador': id_doador}
        q = "SELECT sex FROM dim_doador WHERE id_doador = %(id_doador)d" % pdict
        rows = self.query_exec(q)
        return rows[0][0]


class Donor():

    def __init__ (self, doador_id):
        self.id = doador_id
        self.donations = []
        
        db = SingletonDB()
        self.sex = db.donor_sex(self.id)


    def __str__ (self):
        d = {'donor_id': self.id, 'sex': self.sex, 'donations': self.donations}
        s = str(d) + "\n"
        # truncate on 2 decimals
        s = re.sub(r'([0-9]+\.[0-9]{2})[0-9]+', r'\1', s)
        return s

    def extract_time_series (self):
        db = SingletonDB()

        if not self.donations:
            for row in db.donations_by_donor(self.id):
                date_ = date(row[2], row[3], row[4])
                date_offset = (date_ - db.meta['start_date']).days
                hemat = row[5]
                hemog = float(row[6])
                type = str(row[8])
                d = (date_offset, hemat, hemog, type)
                self.donations.append(d)

    def set_time_series (self, donations):
        self.donations = donations
