# -- coding: utf-8 --

from __future__ import division

from settings import *
from src.calculator import CALC_MAX_ALPHA, CALC_MIN_ALPHA

import matplotlib.pyplot as plt
import numpy as np
import scipy.stats as stats

from drawer import Drawer


class AlphasDrawerComponent(Drawer):
    granularity = "day"

    def image(self):
        plt.clf()
        self.fig = plt.figure()
        self.plot(self.fig)
        return self.save()

    def save(self):
        ax = self.fig.add_subplot(111)
        # put axis labels
        un = "HT" if CALC_USE_HT_METRIC else "HB"
        ylabel = r"$\Delta$%s / %s" % (un, self.granularity)
        ax.set_ylabel(ylabel)
        # grid on
        ax.grid(True)

        plt.savefig(self.filepath)
        return self.filepath


class AlphasDrawerComposition(AlphasDrawerComponent):

    def __init__(self, paths, filename):
        super(AlphasDrawerComposition, self).__init__(paths, filename)
        self._components = []

    def add(self, c):
        self._components.append(c)

    def ordered(self):
        for c in self._components:
            c.ordered()
        return self

    def yearly(self):
        if self.granularity != "year":
            for c in self._components:
                c.yearly()
            self.granularity = "year"
        return self

    def plot(self, fig):
        legend = { 'plots': [], 'names': [] }
        for ad in self._components:
            plot = ad.plot(fig)
            legend['plots'].append(plot)
            legend['names'].append(ad.id)
        fig.legend(tuple(legend['plots']), legend['names'], 'upper right')

    def images(self):
        img_files = []
        for ad in self._components:
            img_files.append(ad.image())

        img_files.append(self.image())
        return img_files


class AlphasHistogramComposition(AlphasDrawerComposition):

    def plot(self, fig=None):
        self.fig, dummy = plt.subplots()
        bins = {}
        counts = {}
        for ad in self._components:
            counts[ad], bins[ad], patches = dummy.hist(ad.alphas, bins=30,
                range=(CALC_MIN_ALPHA, CALC_MAX_ALPHA))

        self.fig, ax = plt.subplots()
        legend = { 'plots': [], 'names': [] }
        for ad in self._components:
            alphas = []
            bin_centers = 0.5 * np.diff(bins[ad]) + bins[ad][:-1]
            for count, x in zip(counts[ad], bin_centers):
                percent_count = int((100 * float(count) / counts[ad].sum()) * 100)
                alphas.extend([x] * percent_count)

            c, b, p = ax.hist(alphas, align="mid", histtype="stepfilled", bins=30,
                            range=(CALC_MIN_ALPHA, CALC_MAX_ALPHA), alpha=0.5, color=ad.format)

            legend['plots'].append(p)
            legend['names'].append(ad.id)
        self.fig.legend(tuple(legend['plots']), legend['names'], 'upper right')

        #raise Exception("Does not make sense multiples histograms in one graph!")

    def images(self):
        img_files = []
        for ad in self._components:
            img_files.append(ad.image())
        return img_files


class AlphasBoxplotComposition(AlphasDrawerComposition):

    def plot(self, fig=None):
        ylist = []
        positions = []
        for ad in self._components:
            ylist.append(ad.alphas)
            positions.append(ad.id)
        plt.boxplot(ylist, sym='', positions=positions)
        ax = self.fig.add_subplot(111)
        xlabel = "#donations / year"
        ax.set_xlabel(xlabel)


class AlphasDrawer(AlphasDrawerComponent):
    granularity = "day"

    def __init__(self, paths, alphas, filepattern, id="", format='b'):
        super(AlphasDrawer, self).__init__(paths, filepattern, id=id)
        self.alphas = alphas
        self.class_num = id
        self.format = format

    def ordered(self):
        self.alphas = sorted(self.alphas)
        return self

    def yearly(self):
        if self.granularity != "year":
            self.alphas = [ a * 365 for a in self.alphas ]
            self.granularity = "year"
        return self

    def plot(self, fig):
        ax = fig.add_subplot(111)
        counts = range(len(self.alphas))
        return ax.plot(counts, self.alphas, self.format)


class AlphasHistogramDrawer(AlphasDrawer):

    def save(self):
        ax = self.fig.add_subplot(111)
        # grid on
        ax.grid(True)

        plt.savefig(self.filepath)
        return self.filepath

class AlphasHistogram(AlphasHistogramDrawer):

    def plot(self, fig=None):
        n, bins, patches = plt.hist(self.alphas, bins=30,
            range=(CALC_MIN_ALPHA, CALC_MAX_ALPHA), align="left")

# attempt to group alphas like a normal distribution
class AlphasNormalFit(AlphasHistogramDrawer):

    def plot(self, fig=None):
        stats.probplot(self.alphas, plot=plt)

class AlphasClassificationHistogram(AlphasHistogramDrawer):
    def plot(self, fig=None):
        from matplotlib.ticker import FormatStrFormatter

        m = np.average(self.alphas) # mean
        s = np.std(self.alphas) # standard deviation

        #bins = [ CALC_MIN_ALPHA, m-3*s, m-2*s, m-s, m, m+s, m+2*s, m+3*s, CALC_MAX_ALPHA ]
        bins = [ m-3*s, m-2*s, m-s, m, m+s, m+2*s, m+3*s ]

        self.fig, ax = plt.subplots()
        counts, bins, patches = ax.hist(self.alphas, bins=bins, align="mid")

        # Set the ticks to be at the edges of the bins.
        ax.set_xticks(bins)
        # Set the xaxis's tick labels to be formatted with 1 decimal place...
        ax.xaxis.set_major_formatter(FormatStrFormatter('%0.3f'))

        patches[0].set_facecolor('#ff0000')
        patches[1].set_facecolor('#ff9900')
        patches[2].set_facecolor('#ffff00')
        patches[3].set_facecolor('#ffff00')
        patches[4].set_facecolor('#99ff00')
        patches[5].set_facecolor('#00ff00')

        # Label the raw counts and the percentages below the x-axis...
        bin_centers = 0.5 * np.diff(bins) + bins[:-1]
        for count, x in zip(counts, bin_centers):
            # Label the percentages
            percent = '%0.1f%%' % (100 * float(count) / counts.sum())
            ax.annotate(percent, xy=(x, 0), xycoords=('data', 'axes fraction'),
                xytext=(0, -22), textcoords='offset points', va='top', ha='center')
